% PaperPlots.m
%
% This file generates the figures shown in the following submission:
%
% B. Hu and P. Seiler, "A Probabilistic Method for Certification of
% Analytically Redundant Systems, submitted to the International Journal
% of Applied Mathematics and Computer Science, 2014.
%

%% Initialize Parameters

% Window Sizes
dt = 0.01;              % Sample time, sec
N = 3600/dt;            % Number of samples per hour
N0 = 20;                % Number of samples in detection window.

% Per-frame actuator failure rate (geometric failure distribution)
MTBF = 3.6e6;           % Mean-time between failure, sec
q = 1-exp(-dt/MTBF);    % Geometric single-step failure rate, unitless
qhat = 1-(1-q)^N;       % Geometric N-step failure rate, unitless

%% System reliability PSN vs. normalized fault level and threshold.
fdata = [1 10];       % Normalized fault levels f/sigma, unitless
Hdata = 0:0.01:20;      % Normalized threshold H/sigma, unitless

fdata = 1:0.1:10;

Nf = numel(fdata);
NH = numel(Hdata);
PSN = zeros(Nf,NH);
PFdata = zeros(Nf,NH);
PDdata = zeros(Nf,NH);
for i1=1:Nf
    for i2=1:NH
        % Set normalized fault level and threshold.
        f = fdata(i1);
        H = Hdata(i2);
        
        % Compute per-frame false alarm and detection probability
        rt2 = sqrt(2);
        PF = 1 - erf(H/rt2);
        PD = 1 -( erf((H+f)/rt2) - erf((-H+f)/rt2) )/2;
        
        % Store per-frame false alarm and detection probability
        PFdata(i1,i2) = PF;
        PDdata(i1,i2) = PD;
        
        % Compute probabilities of basic events
        PMN = q*(1-PD)^(N0)*(1-(1-PF)^N*(1-q)^N) / (1-(1-PF)*(1-q));
        PFN = (1-(1-PF)^N)*(1-qhat)*qhat;
        PDNEN = (qhat-PMN)*qhat;
        
        % Compute total system failure rate
        PSN(i1,i2) = PMN + PFN + PDNEN;        
    end
end

% Compute multiple (N-step) FDI performance probabilities
PF = PFdata;
PD = PDdata;
PFhat = 1-(1-PFdata).^N;
PDhat = 1-(1-PDdata).^N0;

% Plot PSN vs threshold for different fault sizes
figure(1);
semilogy(Hdata,PSN(1,:),'b',Hdata,PSN(end,:),'r--');
xlabel('H/\sigma');
ylabel('P_{S,N}')
grid on;
legend('f/\sigma=1','f/\sigma=10')
set(gca,'Xtick',0:4:20,'XMinorGrid','off');
set(gca,'Ytick',10.^(-6:-2),'YMinorGrid','off');
if exist('garyfyFigure','file')>0
    % Function to clean up plot for paper submissions
    garyfyFigure;
end


%% PSN vs. normalized fault level for optimal threshold

% Determine optimal threshold and performance
[PSNOpt,HOptidx] = min(PSN,[],2);
HOpt = Hdata(HOptidx);

% Plot optimal PSN vs normalized fault size
figure(2);
subplot(211)
semilogy(fdata,PSNOpt);
xlabel('f/\sigma');
ylabel('P_{S,N}^*')
grid on;
set(gca,'XMinorGrid','off');
set(gca,'Ytick',10.^(-6:-3),'YMinorGrid','off');

% Plot optimal threshold vs normalized fault size
subplot(212)
plot(fdata,HOpt);
xlabel('f/\sigma');
ylabel('H^*')
axis([fdata(1) fdata(end) 5 9]);
grid on;
if exist('garyfyFigure','file')>0
    % Function to clean up plot for paper submissions
    garyfyFigure;
end

%% Comparison with ROC curves

% Select data at a particular fault level 
fidx = 26;
PFi = PF(fidx,:);
PDi = PD(fidx,:);
PFhati = PFhat(fidx,:);
PDhati = PDhat(fidx,:);

% Plot ROC curves for single and multiple frames
figure(3)
plot(PFi,PDi,'b',PFhati,PDhati,'r--'); %,PFi,PFi,'k:');
legend('Single-frame ROC','Multiple-frame ROC');
xlabel('False Alarm Probability');
ylabel('Detection Probability');

% Overlay the data at the optimal threshold
hold on;
idx = HOptidx(fidx);
plot(PFi(idx),PDi(idx),'bs',PFhati(idx),PDhati(idx),'rs','MarkerSize',10)
grid on;
hold off;
if exist('garyfyFigure','file')>0
    % Function to clean up plot for paper submissions
    garyfyFigure;
end
